from dash import Input  # type: ignore


def register_callbacks(app, gt1000):
    for n in range(1, 5):
        app.callback(
            Input(f"fx{n}_on", "n_clicks"),
            prevent_initial_call=True,
        )(lambda n_clicks, fx_num=n: send_fx_command(gt1000, fx_num, "on", n_clicks))

        app.callback(
            Input(f"fx{n}_off", "n_clicks"),
            prevent_initial_call=True,
        )(lambda n_clicks, fx_num=n: send_fx_command(gt1000, fx_num, "off", n_clicks))


def send_fx_command(gt1000, fx_num, action, n_clicks):
    if not n_clicks:
        return
    if action == "on":
        gt1000.send_message(gt1000.enable_fx(fx_num))
    else:
        gt1000.send_message(gt1000.disable_fx(fx_num))



from dash import Dash, Input, Output, dcc  # type: ignore
import dash_bootstrap_components as dbc
import argparse
import flask
import signal
import dash
import requests
import threading
import os
from pathlib import Path
from gt1000pilot.shared import (
    gt1000,
    open_gt1000,
    logger,
    buttons_pc_height,
)
from time import sleep

try:
    import tkinter as tk

    cli_only = False
except Exception:
    print("tkinter not installed, running in CLI mode")
    cli_only = True


def launch(app):
    app.layout = dbc.Container(
        fluid=True,  # Ensure the container takes up the full width of the viewport
        children=[
            # Top navigation bar (20% height)
            dbc.Row(
                dbc.Col(
                    id="button-grid",
                    children=[
                        dcc.Link(
                            id=f"page_{page['name']}",
                            children=page["name"].upper(),
                            href=page["relative_path"],
                            style={
                                "backgroundColor": "white",
                                "display": "flex",
                                "font-weight": "bold",
                                "justify-content": "center",
                                "align-items": "center",
                                "textDecoration": "none",
                                "color": "black",
                                "padding": "0.5rem",
                                "height": "100%",
                            },
                        )
                        for i, page in enumerate(dash.page_registry.values())
                    ],
                    style={
                        "display": "grid",
                        "grid-template-columns": f"repeat({len(dash.page_registry)}, 1fr)",
                        "width": "100%",
                        "height": "100%",  # Ensure full height of this section is used
                        "gap": "0",
                        "box-sizing": "border-box",
                    },
                ),
                style={
                    "height": "20vh",
                    "width": "100vw",
                },
            ),
            # Middle section for buttons (70% height)
            dbc.Row(
                dbc.Col(
                    children=dash.page_container,
                    style={
                        "display": "flex",
                        "flex-direction": "column",
                        "flex-grow": "1",
                        "justify-content": "center",  # Center content vertically
                        "align-items": "center",  # Center content horizontally
                        "width": "100%",
                        "overflow-y": "auto",  # Allow scrolling if content overflows
                        "padding": "1rem",  # Optional padding
                        "box-sizing": "border-box",
                        "height": f"{buttons_pc_height}vh",
                        "flex": "1",
                    },
                ),
                style={
                    "height": f"{buttons_pc_height}vh",
                    "flex": "1",
                    "width": "100vw",
                    "flex-grow": "1",
                },
            ),
            # Bottom section for text (10% height)
            #        dbc.Row(
            #            dbc.Col(
            #                html.P(
            #                    "Some footer text here",
            #                    style={
            #                        "text-align": "center",
            #                        "margin": "0",
            #                        "padding": "1rem",
            #                        "font-size": "1rem",
            #                        "color": "gray",
            #                    }
            #                ),
            #                style={
            #                    "display": "flex",
            #                    "align-items": "center",
            #                    "justify-content": "center",
            #                    "background-color": "#f8f8f8",  # Light background for the footer
            #                    "height": "100%",  # Ensure full height of the allocated 10%
            #                },
            #            ),
            #            style={"height": "10vh"},
            #        ),
        ],
        style={"height": "100vh", "width": "100vw"},
    )

    app.run_server(debug=False, host="0.0.0.0")
    gt1000.stop_refresh_thread()

    # Consolidated callback to handle all link styles
    @app.callback(
        [
            Output(f'page_{page["name"]}', "style")
            for page in dash.page_registry.values()
        ],
        Input("_pages_location", "pathname"),
    )
    def update_all_link_styles(pathname):
        styles = []
        for page in dash.page_registry.values():
            if pathname == page["relative_path"]:
                styles.append(
                    {
                        "backgroundColor": "black",
                        "display": "flex",
                        "justify-content": "center",
                        "align-items": "center",
                        "textDecoration": "none",
                        "font-weight": "bold",
                        "color": "white",
                        "padding": "0.5rem",
                        "height": "100%",
                    }
                )
            else:
                styles.append(
                    {
                        "backgroundColor": "white",
                        "display": "flex",
                        "justify-content": "center",
                        "align-items": "center",
                        "textDecoration": "none",
                        "color": "black",
                        "font-weight": "bold",
                        "padding": "0.5rem",
                        "height": "100%",
                    }
                )
        return styles


class AppLauncher(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("GT-1000PILOT Launcher")
        self.geometry("300x380")
        self.app_thread = None
        self.polling_thread = None
        self.stop_polling = threading.Event()

        # Load and resize the logo image
        logo_path = Path("gt1000pilot") / "logo.png"
        self.original_logo = tk.PhotoImage(file=logo_path)
        self.logo = self.original_logo.subsample(
            3, 3
        )  # Adjust the subsample values as needed

        # Display the image in a Label
        self.logo_label = tk.Label(self, image=self.logo)
        self.logo_label.pack(pady=10)

        # Status Label
        self.status_label = tk.Label(self, text="Application not started.", fg="red")
        self.status_label.pack(pady=10)

        # Start Button
        self.start_button = tk.Button(
            self, text="Start Application", command=self.start_app
        )
        self.start_button.pack(pady=10)

        # Quit
        self.stop_button = tk.Button(
            self, text="Quit", command=self.stop_app, state=tk.NORMAL
        )
        self.stop_button.pack(pady=10)

    def start_app(self):
        if self.app_thread is None:
            # self.stop_button.config(state=tk.NORMAL)
            self.start_button.config(state=tk.DISABLED)
            self.status_label.config(text="Loading application...", fg="orange")
            self.stop_polling.clear()
            self.polling_thread = threading.Thread(target=self.poll_server)
            self.polling_thread.start()

            # This needs to start before the Dash app
            if not open_gt1000():
                logger.error("Failed to open GT1000 communication")
                self.status_label.config(text="Failed to open GT-1000", fg="red")
                return

            # Apparently the only way to kill the GUI and dash app
            # looking for a better solution !
            server = flask.Flask(__name__)

            @server.route("/shutdown", methods=["POST"])
            def shutdown():
                pid = os.getpid()
                os.kill(pid, signal.SIGINT)

            # This cannot live in a thread
            app = Dash(
                __name__,
                use_pages=True,
                pages_folder="pages",
                server=server,
                external_stylesheets=[dbc.themes.BOOTSTRAP],
            )
            self.app_thread = threading.Thread(target=launch, args=(app,))
            self.app_thread.start()

    def stop_app(self):
        if self.app_thread:
            requests.post("http://localhost:8050/shutdown")
            self.app_thread.join()
        pid = os.getpid()
        os.kill(pid, signal.SIGINT)

    def poll_server(self):
        while not self.stop_polling.is_set():
            try:
                response = requests.get("http://localhost:8050")
                if response.status_code == 200:
                    self.status_label.config(text="Application is running. Connect to http://<your-ip>:8050", fg="green")
                    return
            except requests.ConnectionError:
                pass
            sleep(1)  # Poll every second
        self.status_label.config(text="Failed to start application.", fg="red")

    def on_closing(self):
        self.stop_app()
        self.destroy()

def cli_launch():
    while not open_gt1000():
        logger.error("Failed to open GT1000 communication")
        sleep(1)
    app = Dash(
            __name__,
            use_pages=True,
            pages_folder="pages",
            external_stylesheets=[dbc.themes.BOOTSTRAP],
            )
    launch(app)

def gui_launch():
    launcher = AppLauncher()
    launcher.protocol("WM_DELETE_WINDOW", launcher.on_closing)
    launcher.mainloop()

if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument("--gui", action="store_true")
    args = parser.parse_args()

    if cli_only or args.gui is False:
        cli_launch()
    else:
        gui_launch()



from pygt1000 import GT1000
import logging


menu_color1 = "#ABBACC"
menu_color2 = "#C1C9CB"
#off_color = "#F2E9D8"
off_color = "white"
on_color = "#2D8C2A"

buttons_pc_height = 70

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

gt1000 = GT1000()
gt1000.dash_effects = {}

# Mac and Linux default portname prefixes
known_default_portname_prefixes = ["GT-1000", "GT-1000:GT-1000 MIDI 1"]

def open_gt1000():
    opened = False
    for portname in known_default_portname_prefixes:
        if not gt1000.open_ports(portname=portname):
            continue
        opened = True
    if not opened:
        return False
    gt1000.refresh_state()
    gt1000.start_refresh_thread()
    return True



#!/usr/bin/env python3

from pygt1000 import GT1000
from time import sleep

gt1000 = GT1000()
# gt1000.open_ports()
# gt1000.fetch_patch_names()

# sleep(2)
# gt1000.get_fx1()

while True:
    sleep(1)



import dash
from dash import html, dcc, ctx, Input, Output, callback, get_app
from datetime import datetime
from time import sleep

from gt1000pilot.shared import gt1000, off_color, on_color, logger
from gt1000pilot.pages.pages_common import (
    send_fx_state_command,
    register_callbacks,
    refresh_all_effects,
    generate_buttons,
    serve_layout,
    callbacks_registered,
)

dash.register_page(__name__, path="/comp")

state_key = "comp"
callbacks_registered[state_key] = False


@callback(
    Output(f"{state_key}_buttons", "children"),
    Input("interval-component", "n_intervals"),
)
def update_metrics(n):
    refresh_all_effects(state_key)
    return generate_buttons(state_key)


layout = serve_layout(state_key)



import dash
from dash import html, dcc, ctx, Input, Output, callback, get_app, State
import dash_bootstrap_components as dbc
from datetime import datetime
from time import sleep

from gt1000pilot.shared import gt1000, off_color, on_color, logger, buttons_pc_height

last_action_ts = None

callbacks_registered = {}


def get_icon(fx_type):
    prefix = "/assets/"
    icons = {
        "dist": "stompbox-dist.png",
        "eq": "stompbox-eq.png",
        "fx": "stompbox-fx.png",
        "comp": "stompbox-comp.png",
    }
    if fx_type in icons:
        return f"{prefix}{icons[fx_type]}"
    return f"{prefix}stompbox-fx.png"


def register_callbacks(app, fx_type):
    for n in range(1, len(gt1000.dash_effects[fx_type]) + 1):
        app.callback(
            Output(f"{fx_type}_toggle_fx{n}", "style"),
            Input(f"{fx_type}_toggle_fx{n}", "n_clicks"),
            prevent_initial_call=True,
        )(lambda n_clicks, fx_num=n: send_fx_state_command(fx_type, fx_num, n_clicks))

        # Slider callback
        for s in ["slider1", "slider2"]:
            slider_dict = gt1000.dash_effects[fx_type][n - 1][s]
            if slider_dict is not None:
                slider_id = f'slider_{fx_type}{n}_{slider_dict["label"]}'

                app.callback(
                    #                        Output(f"{fx_type}_toggle_fx{n}", "style"),  # Example output, adjust to your needs
                    Input(slider_id, "value"),
                    prevent_initial_call=True,
                )(
                    lambda value,
                    fx_type=fx_type,
                    fx_id=n,
                    slider=s: handle_slider_change(value, fx_type, fx_id, slider)
                )


def refresh_all_effects(fx_type):
    global callbacks_registered
    gt1000_ready = True
    try:
        if fx_type not in gt1000.get_state():
            current_state = {fx_type: []}
            gt1000_ready = False
        current_state = gt1000.get_state()
    except Exception:
        # Catch all to avoid dying on unhandled exceptions
        logger.exception("Exception caught for toggle_fx_state")
    # If we clicked on a button but the current_state from the pedal wasn't
    # sync'ed yet, we want to keep our old state otherwise the pedal color will
    # go back to its previous state.
    if (
        last_action_ts is None
        or current_state["last_sync_ts"][fx_type] > last_action_ts
    ):
        gt1000.dash_effects[fx_type] = current_state[fx_type]
    for i in range(len(gt1000.dash_effects[fx_type])):
        if gt1000.dash_effects[fx_type][i]["state"] == "OFF":
            gt1000.dash_effects[fx_type][i]["color"] = off_color
        else:
            gt1000.dash_effects[fx_type][i]["color"] = on_color

    if gt1000_ready and not callbacks_registered[fx_type]:
        register_callbacks(get_app(), fx_type)
        callbacks_registered[fx_type] = True


def build_one_slider(fx_type, fx_id, slider):
    if slider is None:
        return html.Div()
    # Special case for EQ, we could technically find this automatically in the spec json
    if fx_type == "eq":
        marks = {12: "-20dB",
               32: "0dB",
               52: "+20dB"}
    else:
        marks = {}
    return html.Div(
        [
            html.Label(
                slider["label"], style={"text-align": "center", "width": "100%"}
            ),
            dcc.Slider(
                min=slider["min"],
                max=slider["max"],
                value=slider["value"],
                id=f'slider_{fx_type}{fx_id}_{slider["label"]}',
                marks=marks
            ),
        ]
    )


def build_grid(fx_type):
    grid = []
    num_effects = len(gt1000.dash_effects[fx_type])
    col_width = int(12 / num_effects)  # Column width based on number of effects

    for n in range(1, num_effects + 1):
        slider1_dict = gt1000.dash_effects[fx_type][n - 1]["slider1"]
        slider2_dict = gt1000.dash_effects[fx_type][n - 1]["slider2"]

        sliders = html.Div(
            [
                build_one_slider(fx_type, n, slider1_dict),
                build_one_slider(fx_type, n, slider2_dict),
            ],
            style={
                "width": "100%",
                "padding": "10px 0",
            },  # Ensure full width and some spacing
        )

        grid.append(
            dbc.Col(
                width=col_width,
                children=[
                    html.Div(
                        [
                            html.Button(
                                id=f"{fx_type}_toggle_fx{n}",
                                children=[
                                    html.Div(
                                        children=[
                                            html.Img(
                                                src=get_icon(fx_type),
                                                style={
                                                    "max-width": "80%",  # Ensure the image width is capped at 100% of its container
                                                    "max-height": "80%",  # Ensure the image height is capped at 100% of its container
                                                    "width": "auto",  # Adjust the width to maintain aspect ratio
                                                    "height": "auto",  # Adjust the height to maintain aspect ratio
                                                    "object-fit": "contain",  # Ensure the entire image is visible, scaling as needed
                                                },
                                            ),
                                            html.H2(
                                                id=f"fx{n}_name",
                                                children=gt1000.dash_effects[fx_type][
                                                    n - 1
                                                ]["name"],
                                                style={
                                                    "text-align": "center",
                                                    "margin": "0",
                                                },  # Center text and remove margins
                                            ),
                                        ],
                                        style={
                                            "color": "black",
                                            "text-align": "center",
                                            "height": "100%",
                                            "display": "flex",
                                            "flex-direction": "column",
                                            "justify-content": "center",
                                            "align-items": "center",
                                            "width": "100%",  # Ensure full width for the content within the button
                                        },
                                    )
                                ],
                                n_clicks=0,
                                style={
                                    "backgroundColor": gt1000.dash_effects[fx_type][
                                        n - 1
                                    ]["color"],
                                    "display": "flex",
                                    "flex-direction": "column",
                                    "align-items": "center",
                                    "justify-content": "center",
                                    "width": "100%",  # Ensure button takes full width of column
                                    "height": "100%",  # Ensure button takes full height of column
                                    "box-sizing": "border-box",  # Include padding/border in size calculations
                                    "overflow": "hidden",  # Prevent any content overflow
                                    "textDecoration": "none",
                                },
                            ),
                            sliders,
                        ],
                        style={
                            "display": "flex",
                            "flex-direction": "column",  # Stack button and slider vertically
                            "align-items": "center",
                            "width": "100%",  # Ensure full width of the container
                        },
                    ),
                ],
                style={
                    "padding": "0",  # Remove padding to prevent overflow
                    "display": "flex",  # Flexbox to handle alignment within the column
                    "align-items": "stretch",  # Stretch button to fill column height
                },
            )
        )

    return grid


def generate_buttons(fx_type):
    grid = build_grid(fx_type)
    return html.Div(
        children=[
            dbc.Row(
                id="button_grid_content",
                children=grid,
                style={
                    "display": "flex",
                    "flex-wrap": "nowrap",  # Ensure items do not wrap to a new row
                    "justify-content": "space-evenly",  # Evenly space out the columns
                    "grid-template-columns": "repeat(auto-fill, minmax(100px, 1fr))",
                    "align-items": "stretch",  # Make sure all columns stretch to the same height
                    "overflow": "hidden",  # Prevent scrollbars
                    "width": "100%",
                    "height": "100%",  # Ensure the grid takes full height of its container
                },
            ),
        ],
        style={
            "display": "flex",
            "flex-direction": "column",
            "height": f"{buttons_pc_height}vh",  # This should match the max-height defined earlier
            "overflow": "hidden",  # Prevent scrollbars on the outer container
        },
    )


def serve_layout(fx_type):
    refresh_all_effects(fx_type)
    return html.Div(
        id="button-grid",
        children=[
            dcc.Interval(
                id="interval-component",
                interval=2 * 1000,  # in milliseconds
                n_intervals=0,
            ),
            html.Div(id=f"{fx_type}_buttons", children=generate_buttons(fx_type)),
        ],
    )


def send_fx_state_command(fx_type, fx_num, n_clicks):
    if not n_clicks:
        return
    global last_action_ts
    last_action_ts = datetime.now()
    if gt1000.dash_effects[fx_type][fx_num - 1]["state"] == "ON":
        logger.info(f"{fx_type}{fx_num} enabled")
        try:
            gt1000.toggle_fx_state(fx_type, fx_num, "OFF")
        except Exception:
            # Catch all to avoid dying on unhandled exceptions
            logger.exception("Exception caught for toggle_fx_state")
        # optimistically update here
        gt1000.dash_effects[fx_type][fx_num - 1]["state"] = "OFF"
        return {
            "backgroundColor": off_color,
            "display": "flex",
            "flex-direction": "column",
            "align-items": "center",
            "justify-content": "center",
            "width": "100%",  # Ensure button takes full width of column
            "height": "100%",  # Ensure button takes full height of column
            "box-sizing": "border-box",  # Include padding/border in size calculations
            "overflow": "hidden",  # Prevent any content overflow
            "textDecoration": "none",
        }
    else:
        try:
            gt1000.toggle_fx_state(fx_type, fx_num, "ON")
        except Exception:
            # Catch all to avoid dying on unhandled exceptions
            logger.exception("Exception caught for toggle_fx_state")
        logger.info(f"{fx_type}{fx_num} disabled")
        # optimistically update here
        gt1000.dash_effects[fx_type][fx_num - 1]["state"] = "ON"
        return {
            "backgroundColor": on_color,
            "display": "flex",
            "flex-direction": "column",
            "align-items": "center",
            "justify-content": "center",
            "width": "100%",  # Ensure button takes full width of column
            "height": "100%",  # Ensure button takes full height of column
            "box-sizing": "border-box",  # Include padding/border in size calculations
            "overflow": "hidden",  # Prevent any content overflow
            "textDecoration": "none",
        }


def handle_slider_change(value, fx_type, fx_id, slider):
    global last_action_ts
    last_action_ts = datetime.now()
    label = gt1000.dash_effects[fx_type][fx_id - 1][slider]["label"]
    logger.info(f"Slider changed: {fx_type}, {fx_id}, {label}, new value: {value}")
    gt1000.dash_effects[fx_type][fx_id - 1][slider]["value"] = value
    try:
        gt1000.set_fx_value(fx_type, fx_id, label, value)
    except Exception:
        # Catch all to avoid dying on unhandled exceptions
        logger.exception("Exception caught for toggle_fx_state")
